@@ -1,13 +1,16 @@
 from typing import Annotated
 
-from flow_py_sdk import cadence
-from flow_py_sdk.signer import AccountKey
-from flow_py_sdk.tx import Tx
+import flow_py_sdk.cadence as cadence
+from flow_py_sdk.account_key import AccountKey
+from flow_py_sdk.tx import Tx, ProposalKey
 
 
 def create_account_template(
     *,
     keys: list[AccountKey],
+    reference_block_id: bytes = None,
+    payer: cadence.Address = None,
+    proposal_key: ProposalKey = None,
     contracts: dict[Annotated[str, "name"], Annotated[str, "source"]] = None
 ) -> Tx:
     if keys:
@@ -29,20 +32,23 @@ def create_account_template(
     tx = (
         Tx(
             code="""
-            transaction(publicKeys: [String], contracts:{String: String}) {
-                prepare(signer: AuthAccount) {
-                    let acct = AuthAccount(payer: signer)
-
-                    for key in publicKeys {
-                        acct.addPublicKey(key.decodeHex())
-                    }
-
-                    for contract in contracts.keys {
-                        acct.contracts.add(name: contract, code: contracts[contract]!.decodeHex())
+                transaction(publicKeys: [String], contracts:{String: String}) {
+                    prepare(signer: AuthAccount) {
+                        let acct = AuthAccount(payer: signer)
+    
+                        for key in publicKeys {
+                            acct.addPublicKey(key.decodeHex())
+                        }
+    
+                        for contract in contracts.keys {
+                            acct.contracts.add(name: contract, code: contracts[contract]!.decodeHex())
+                        }
                     }
                 }
-            }
-        """
+            """,
+            reference_block_id=reference_block_id,
+            payer=payer,
+            proposal_key=proposal_key,
         )
         .add_arguments(cadence_public_keys)
         .add_arguments(cadence_contracts)
