@@ -4,6 +4,7 @@ from enum import Enum
 import rlp
 
 from flow_py_sdk.cadence import Value, Address, encode_arguments
+from flow_py_sdk.exceptions import NotCadenceValueError, PySDKError
 from flow_py_sdk.frlp import rlp_encode_uint64
 from flow_py_sdk.proto.flow import entities
 from flow_py_sdk.signer import Signer
@@ -26,6 +27,21 @@ class TransactionStatus(Enum):
     TransactionStatusExpired = 5
 
 
+DomainTagLength = 32
+
+
+def _padded_domain_tag(s: str) -> bytes:
+    encoded = s.encode("utf-8")
+    if len(encoded) > DomainTagLength:
+        raise PySDKError(
+            f"domain tag {s} cannot be longer than {DomainTagLength} bytes"
+        )
+    return encoded + bytearray(DomainTagLength - len(s))
+
+
+TransactionDomainTag = _padded_domain_tag("FLOW-V0.0-transaction")
+
+
 class TxSignature(object):
     def __init__(
         self, address: Address, key_id: int, signer_index: int, signature: bytes
@@ -147,7 +163,7 @@ class Tx(object):
             raise Exception(
                 f"The transaction needs [{', '.join(self._missing_fields_for_signing())}] before it can be signed"
             )
-        signature = signer.sign(self.payload_message())
+        signature = signer.sign(self.payload_message(), TransactionDomainTag)
         signer_index = self._signer_list().index(address)
         ts = TxSignature(address, key_id, signer_index, signature)
         self.payload_signatures.append(ts)
@@ -160,7 +176,7 @@ class Tx(object):
             raise Exception(
                 f"The transaction needs [{', '.join(self._missing_fields_for_signing())}] before it can be signed"
             )
-        signature = signer.sign(self.envelope_message())
+        signature = signer.sign(self.envelope_message(), TransactionDomainTag)
         signer_index = self._signer_list().index(address)
         ts = TxSignature(address, key_id, signer_index, signature)
         self.envelope_signatures.append(ts)
@@ -171,6 +187,9 @@ class Tx(object):
         return self
 
     def add_arguments(self, *args: Value) -> "Tx":
+        for arg in args:
+            if not isinstance(arg, Value):
+                raise NotCadenceValueError.from_value(arg)
         self.arguments.extend(args)
         return self
 
